# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 E-MIPS (http://www.e-mips.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from datetime import date
from openerp.tests import common
from openerp import netsvc
from openerp.osv.osv import except_osv
from functools import partial
from openerp.addons.service.tests.test_suitcase import CommonSuitcaseFunc
import unittest2


class TestServiceOrder(common.TransactionCase):
    def setUp(self):
        super(TestServiceOrder, self).setUp()
        cr, uid, context = self.cr, self.uid, {}

        # Datos a utilizar
        data_model = self.registry('ir.model.data')
        self.get_ref = partial(data_model.get_object_reference,
                               self.cr, self.uid)

        self.common_suitcase_func = CommonSuitcaseFunc(cr, uid, self.registry)

        self.product_product = self.registry('product.product')
        self.service_order_model = self.registry('service.order')
        self.material_line_model = self.registry('service.order.material.line')
        self.material_line_used_model = self.registry('service.order.material.line.used')
        self.material_quantity_model = self.registry('service.material.quantity')
        self.suitcase_model = self.registry('stock.suitcase')

        __, self.warehouse_id = self.get_ref('stock', 'warehouse0')
        __, self.location_components_id = self.get_ref('stock', 'stock_location_components')
        __, self.partner_id = self.get_ref('base', 'res_partner_7')
        __, self.sector_lab_id = self.get_ref('jenck_partner', 'sector_lab')
        __, self.employee1_id = self.get_ref('hr', 'employee_fp')
        __, self.employee_lab_id = self.get_ref('jenck_partner', 'employee_lab1')
        __, self.type_maintenance_id = self.get_ref('service', 'service_type_maintenance')
        __, self.type_installation_id = self.get_ref('service', 'service_type_installation')
        __, self.system_1_id = self.get_ref('service', 'system_1')
        __, self.component_1_id = self.get_ref('service', 'component_1')
        __, self.product_1_id = self.get_ref('product', 'product_product_13')
        __, self.product_2_id = self.get_ref('product', 'product_product_14')
        __, self.product_3_id = self.get_ref('product', 'product_product_15')


    def _service_wkf(self, order, action):
        wf_service = netsvc.LocalService("workflow")
        wf_service.trg_validate(self.uid, 'service.order',
                                order.id, action, self.cr)
        order.refresh()

    def _create_maintenance_order(self):
        cr, uid = self.cr, self.uid

        order_id = self.service_order_model.create(cr, uid, dict(
                    name="/",
                    type=self.type_maintenance_id,
                    partner_id=self.partner_id,
                    system_id=self.system_1_id,
                    lot_ids=[(4, self.component_1_id, 0)],
                    initial_survey="Survey test",
                    sector_id=self.sector_lab_id,
                    employee_sector_id=self.employee_lab_id,
                    ))

        return self.service_order_model.browse(cr, uid, order_id)

    def _add_material_to_order(self, order, product_id, qty):
        cr, uid = self.cr, self.uid

        line_vals = {}
        vals = self.material_line_model.onchange_product_id(
                cr, uid, [], product_id, qty, None)

        line_vals.update(vals['value'])

        vals = self.material_line_model.onchange_warehouse_id(
               cr, uid, [], self.warehouse_id)

        line_vals.update(vals['value'])

        line_vals['quantity'] = qty
        line_vals['warehouse_id'] = self.warehouse_id
        line_vals['product_id'] = product_id

        # Agregamos a la order
        self.service_order_model.write(cr, uid, order.id,
            {'material_line_ids': [(0, 0, line_vals)]})

        return order.refresh()

    def _add_used_material_to_order(self, order, suitcase_id, product_id, qty):
        cr, uid = self.cr, self.uid

        line_vals = {}
        vals = self.material_line_used_model.onchange_product_id(
                cr, uid, [], product_id, qty, None)

        line_vals.update(vals['value'])

        vals = self.material_line_used_model.onchange_warehouse_id(
               cr, uid, [], self.warehouse_id)

        line_vals.update(vals['value'])

        line_vals['quantity'] = qty
        line_vals['warehouse_id'] = self.warehouse_id
        line_vals['product_id'] = product_id
        line_vals['suitcase_id'] = suitcase_id
        line_vals['state'] = 'draft'

        # Agregamos a la order
        self.service_order_model.write(cr, uid, order.id,
            {'material_line_used_ids': [(0, 0, line_vals)]})

        return order.refresh()

    def _create_inform_materials(self, order, context):
        cr, uid = self.cr, self.uid

        res = self.material_quantity_model.default_get(cr, uid,
                ['line_ids', 'scrap_location_id'], context)
        vals = {'scrap_location_id': res['scrap_location_id']}
        vals['line_ids'] = map(lambda x: (0, 0, x), res['line_ids'])

        mat_qty_id = self.material_quantity_model.create(cr, uid, vals, context)
        return self.material_quantity_model.browse(cr, uid, mat_qty_id)

    def test_01_service_order_create(self):
        """Testeamos creacion de parte de servicio."""
        cr, uid = self.cr, self.uid

        order_id = self.service_order_model.create(cr, uid, dict(
                name="/",
                type=self.type_installation_id,
                partner_id=self.partner_id,
                initial_survey="Survey test",
                system_id=self.system_1_id,
                sector_id=self.sector_lab_id,
                employee_sector_id=self.employee_lab_id,
                ))

        order = self.service_order_model.browse(cr, uid, order_id)
        self.assertEquals(order.name, "/")

        self._service_wkf(order, 'service_confirm')
        self.assertEquals(order.state, 'confirmed')

    def test_02_service_order_create_mandatory(self):
        """Testeamos creacion de parte de servicio, obligatorio llenado de Sistema."""
        cr, uid = self.cr, self.uid

        order_id = self.service_order_model.create(cr, uid, dict(
                name="/",
                type=self.type_maintenance_id,
                partner_id=self.partner_id,
                initial_survey="Survey test",
                sector_id=self.sector_lab_id,
                employee_sector_id=self.employee_lab_id,
                ))

        order = self.service_order_model.browse(cr, uid, order_id)
        self.assertEquals(order.name, "/")

        with self.assertRaises(except_osv):
            self._service_wkf(order, 'service_confirm')

        # Seteamos el sistema y un componente
        order_id = self.service_order_model.create(cr, uid, dict(
                name="/",
                type=self.type_maintenance_id,
                partner_id=self.partner_id,
                system_id=self.system_1_id,
                lot_ids=[(4, self.component_1_id, 0)],
                initial_survey="Survey test",
                sector_id=self.sector_lab_id,
                employee_sector_id=self.employee_lab_id,
                ))

        order = self.service_order_model.browse(cr, uid, order_id)
        order.refresh()
        self._service_wkf(order, 'service_confirm')
        self.assertEquals(order.state, 'confirmed')

    #def test_05_order_resolved_by_phone(self):
        # TODO

    #def test_07_order_planificate(self):
        # TODO

    def test_10_order_give_materials_inform(self):
        """Testeamos la entrega de materiales en el parte e informamos sin dejar remanente."""
        cr, uid = self.cr, self.uid

        # Creamos un parte de servicio y lo confirmamos
        order = self._create_maintenance_order()
        self._service_wkf(order, 'service_confirm')
        self.assertEquals(order.state, 'confirmed')

        # Agregamos líneas de materiales
        self._add_material_to_order(order, self.product_1_id, 10.0)
        #self._add_material_to_order(order, self.product_2_id, 15.0)

        #self.assertEquals(len(order.material_line_ids), 2)
        self.assertEquals(len(order.material_line_ids), 1)
        line0 = order.material_line_ids[0]
        #line1 = order.material_line_ids[1]
        self.assertEquals(line0.state, 'draft')
        self.assertFalse(line0.picking_id)

        # Entregamos los materiales
        self.service_order_model.send_materials(cr, uid, [order.id])
        order.refresh()

        self.assertTrue(line0.picking_id)
        #self.assertTrue(line1.picking_id)
        #self.assertEquals(line0.picking_id, line1.picking_id)
        picking = line0.picking_id

        self.assertEquals(picking.state, 'assigned')
        self.assertEquals(picking.type, 'out')

        # Informamos el uso de los materiales
        context = {'active_model': 'service.order',
                   'active_id': order.id,
                   'active_ids': [order.id]}

        wiz_inform = self._create_inform_materials(order, context)

        line = wiz_inform.line_ids[0]
        qty = line.qty_remaining
        qty_used = qty-3
        qty_scrap = 1
        qty_returned = 2
        line.write({'qty_used': qty_used,
                    'qty_returned': qty_returned,
                    'qty_scrap': qty_scrap})

        self.material_quantity_model.done(cr, uid, [wiz_inform.id], context)

        # Asserts varios
        picking.refresh()
        self.assertEquals(picking.state, 'done')
        self.assertEquals(len(picking.move_lines), 3)

        for line in picking.move_lines:
            if line.state == 'cancel':
                self.assertEquals(line.product_qty, qty_returned)
            elif line.state == 'done':
                if line.location_dest_id.scrap_location:
                    self.assertEquals(line.product_qty, qty_scrap)
                else:
                    self.assertEquals(line.product_qty, qty_used)

        self.assertEquals(len(order.material_line_ids), 2)
        self.assertEquals(len(order.material_line_used_ids), 1)

    def test_12_order_remaining_materials(self):
        """Testeamos la informacion de materiales pero dejando un remanente y luego lo terminamos de informar."""
        cr, uid = self.cr, self.uid

        # Creamos un parte de servicio y lo confirmamos
        order = self._create_maintenance_order()
        self._service_wkf(order, 'service_confirm')
        self.assertEquals(order.state, 'confirmed')

        # Agregamos líneas de materiales
        self._add_material_to_order(order, self.product_1_id, 10.0)
        #self._add_material_to_order(order, self.product_2_id, 15.0)

        #self.assertEquals(len(order.material_line_ids), 2)
        self.assertEquals(len(order.material_line_ids), 1)
        line0 = order.material_line_ids[0]
        #line1 = order.material_line_ids[1]
        self.assertEquals(line0.state, 'draft')
        self.assertFalse(line0.picking_id)

        # Entregamos los materiales
        self.service_order_model.send_materials(cr, uid, [order.id])
        order.refresh()

        self.assertTrue(line0.picking_id)
        #self.assertTrue(line1.picking_id)
        #self.assertEquals(line0.picking_id, line1.picking_id)
        picking = line0.picking_id

        self.assertEquals(picking.state, 'assigned')
        self.assertEquals(picking.type, 'out')

        # Informamos el uso de los materiales
        context = {'active_model': 'service.order',
                   'active_id': order.id,
                   'active_ids': [order.id]}

        wiz_inform = self._create_inform_materials(order, context)

        line = wiz_inform.line_ids[0]
        qty = line.qty_remaining
        qty_used = qty-3
        qty_scrap = 1
        qty_returned = 0
        qty_remaining = qty - qty_used - qty_scrap - qty_returned
        line.write({'qty_used': qty_used,
                    'qty_returned': qty_returned,
                    'qty_scrap': qty_scrap})

        self.material_quantity_model.done(cr, uid, [wiz_inform.id], context)

        # Asserts varios
        picking.refresh()
        self.assertEquals(picking.state, 'assigned')
        self.assertEquals(len(picking.move_lines), 3)

        for line in picking.move_lines:
            if line.state == 'cancel':
                self.assertEquals(line.product_qty, qty_returned)
            elif line.state == 'done':
                if line.location_dest_id.scrap_location:
                    self.assertEquals(line.product_qty, qty_scrap)
                else:
                    self.assertEquals(line.product_qty, qty_used)
            else:
                self.assertEquals(line.product_qty, qty_remaining)

        # Terminamos de informar lo que queda
        context = {'active_model': 'service.order',
                   'active_id': order.id,
                   'active_ids': [order.id]}

        wiz_inform = self._create_inform_materials(order, context)

        line = wiz_inform.line_ids[0]
        qty = line.qty_remaining
        qty_used = 0
        qty_scrap = 0
        qty_returned = qty
        qty_remaining = qty - qty_used - qty_scrap - qty_returned
        line.write({'qty_used': qty_used,
                    'qty_returned': qty_returned,
                    'qty_scrap': qty_scrap})

        self.material_quantity_model.done(cr, uid, [wiz_inform.id], context)

        # Asserts varios
        picking.refresh()
        self.assertEquals(picking.state, 'done')
        self.assertEquals(len(picking.move_lines), 3)
        self.assertEquals(len(order.material_line_ids), 2)
        self.assertEquals(len(order.material_line_used_ids), 1)

    def test_15_order_manual_material_used(self):
        """Testeamos la creacion de materiales usados manualmente."""
        cr, uid = self.cr, self.uid

        # Creamos una suitcase y la confirmamos
        suitcase = self.common_suitcase_func._create_suitcase('Test Suitcase',
                self.employee1_id)
        self.suitcase_model.confirm(cr, uid, [suitcase.id])

        # Llenamos la valija con materiales
        wiz = self.common_suitcase_func._add_materials_to_suitcase(
                suitcase, self.product_1_id,
                self.location_components_id, 10.0)
        suitcase.refresh()

        # El picking no tiene que estar realizado
        pick = wiz.picking_id
        self.assertEquals(pick.state, 'draft')

        self.common_suitcase_func._transfer_picking(pick)
        self.assertEquals(pick.state, 'done')



        # Creamos un parte de servicio y lo confirmamos
        order = self._create_maintenance_order()
        self._service_wkf(order, 'service_confirm')
        self.assertEquals(order.state, 'confirmed')

        # Agregamos líneas de materiales
        self._add_used_material_to_order(order, suitcase.id, self.product_1_id, 3.0)

        self.assertEquals(len(order.material_line_used_ids), 1)

        line0 = order.material_line_used_ids[0]
        self.assertEquals(line0.state, 'used')

        self.assertTrue(line0.picking_id)
        picking = line0.picking_id

        self.assertEquals(picking.type, 'out')
        self.assertEquals(picking.state, 'done')
#
#
