# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2016 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2016 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General
#    Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from openerp.osv import fields, orm, osv
from openerp.tools.translate import _

import logging

_logger = logging.getLogger(__name__)


class AddComponentToOrderWizard(orm.TransientModel):
    _name = 'add.component.to.order.wizard'

    _columns = {
        'component_ids': fields.many2many(
            'service.component',
            'service_component_add_to_order_wizard_rel',
            'order_id', 'wiz_id', 'Components'),
        'system_id': fields.many2one('service.system', 'System'),
    }

    def _get_default_system(self, cr, uid, context=None):
        try:
            res = self._get_order(cr, uid, context=context).system_id.id
        except Exception:
            res = False
        return res

    def _get_default_components(self, cr, uid, context=None):
        try:
            order = self._get_order(cr, uid, context=context)
            res = [comp.id for comp in order.component_ids]
        except Exception:
            res = False
        return res

    _defaults = {
        'system_id': _get_default_system,
        'component_ids': _get_default_components,
    }

    def _get_order(self, cr, uid, context=None):
        if not context:
            return []
        order_obj = self.pool['service.order']
        order_id = context.get('active_ids', False)
        order = order_obj.browse(cr, uid, order_id, context)
        if not order_id or len(order_id) != 1:
            raise osv.except_osv(
                _('Error'), _('Expected only one order in Wizard Call!'))
        if isinstance(order, list):
            order = order[0]
        return order

    def save(self, cr, uid, ids, context=None):
        order_obj = self.pool['service.order']
        component_in_use_ids = []
        component_in_disuse = []
        
        wiz = self.browse(cr, uid, ids[0], context=context)

        for component in wiz.component_ids:
          if component.in_disuse:
            component_in_disuse.append(component.product_id.name)
          else:
            component_in_use_ids.append(component.id)

        if component_in_disuse:
          message = "You can not select these components " + (", ").join(component_in_disuse) + " they are in disuse!" 
          raise osv.except_osv(_('Error'), _(message))

        return order_obj.write(cr, uid, context['active_id'], 
                              {'component_ids': [(6,0,component_in_use_ids)]}, context=context)