# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2016 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2016 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General
#    Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from openerp.osv import fields, orm, osv
from openerp.tools.translate import _

import logging

_logger = logging.getLogger(__name__)


class AddComponentToSystemWizard(orm.TransientModel):
    _name = 'add.component.to.system.wizard'

    _columns = {
        'component_ids': fields.many2many(
            'service.component',
            'service_component_add_to_system_wizard_rel',
            'system_id', 'wiz_id', 'Components'),
        'system_id': fields.many2one('service.system', 'System'),
        'partner_id': fields.many2one('res.partner', 'Partner'),
    }

    def _get_default_system(self, cr, uid, context=None):
        try:
            res = self._get_system(cr, uid, context=context).id
        except Exception:
            res = False
        return res

    def _get_default_partner(self, cr, uid, context=None):
        try:
            res = self._get_system(cr, uid, context=context).partner_id.id
        except Exception:
            res = False
        return res

    def _get_default_components(self, cr, uid, context=None):
        try:
            system = self._get_system(cr, uid, context=context)
            res = [comp.id for comp in system.component_ids]
        except Exception:
            res = False
        return res

    _defaults = {
        'system_id': _get_default_system,
        'partner_id': _get_default_partner,
        'component_ids': _get_default_components,
    }

    def _get_system(self, cr, uid, context=None):
        if not context:
            return []
        system_obj = self.pool['service.system']
        system_id = context.get('active_ids', False)
        system = system_obj.browse(cr, uid, system_id, context)
        if not system_id or len(system_id) != 1:
            raise osv.except_osv(
                _('Error'), _('Expected only one system in Wizard Call!'))
        if isinstance(system, list):
            system = system[0]
        return system

    def save(self, cr, uid, ids, context=None):
        component_obj = self.pool['service.component']
        system = self._get_system(cr, uid, context=context)
        wiz = self.browse(cr, uid, ids[0], context=context)
        component_in_disuse = []

        for component in wiz.component_ids:
          if component.in_disuse:
            component_in_disuse.append(component.product_id.name)

        if component_in_disuse:
          message = "You can not select these components " + ("-").join(component_in_disuse) + " they are in disuse!" 
          raise osv.except_osv(_('Error'), _(message))

        component_obj.write(cr, uid, [c.id for c in system.component_ids], {
            'system_id': False,
        }, context=context)
        component_obj.write(cr, uid, [c.id for c in wiz.component_ids], {
            'system_id': system.id,
        }, context=context)
        return True
